import argparse
import sys
import src.files as files
import src.imports as imports
import src.reset as reset
import src.upload as upload
from src.ui import navigation
from src.ui.OptionSelector import OptionSelector

INVALID_ARGUMENT_TEXT = "Invalid argument given. Must be 'generate', 'upload', 'reset', \
'fiximports', 'unfiximports', or 'help'."

ARGS_DESCRIPTION_TEXT = "Enter command 'generate', 'upload', 'reset', 'fiximports', \
'unfiximports', or 'help'"

HELP_TEXT = "\nPyPI Maker\n----------\n\n\
To run, enter 'pypimaker' followed by a valid command.\n\
Some examples:\n\n\
   pypimaker generate\n\
   pypimaker upload\n\
   pypimaker -r\
\n\nThe possible commands (or their aliases) are the following:\n\n\
   generate  (-g)\n\
   upload    (-u)\n\
   reset     (-r)\n\
   fix       (-f)\n\
   unfix     (-z)\n\
   help      (-h)\n\n\
See the documentation on GitHub or PyPI for more information.\n"

def main(argv=sys.argv):
	parser = argparse.ArgumentParser(description="Example input: pypimaker generate")
	parser.add_argument(
		"command", 
		type=str, 
		help=ARGS_DESCRIPTION_TEXT, 
		nargs=1
	)
	userOption = "help" if len(sys.argv) == 1 else sys.argv[1]
	
	if userOption in ["generate", "-g"]:
		options = OptionSelector()
		files.generate(
			options.filepath,
			options.projectName,
			options.authorsArray,
			options.emailsArray,
			options.correspondingEmail,
			options.githubUsername,
			options.shortDescription,
			options.classifier
		)
	
	elif userOption in ["upload", "-u"]:
		filepath = navigation.getDirectory()
		upload.upload(filepath)
	
	elif userOption in ["reset", "-r"]:
		filepath = navigation.getDirectory()
		reset.reset(filepath)
	
	elif userOption in ["fix", "-f"]:
		filepath = navigation.getDirectory()
		imports.fix(filepath)
	
	elif userOption in ["unfix", "-z"]:
		filepath = navigation.getDirectory()
		imports.unfix(filepath)
	
	elif userOption in ["help", "-h"]:
		print(HELP_TEXT)
	
	else:
		raise Exception(INVALID_ARGUMENT_TEXT)

if __name__ == '__main__':
	main()



import os
import subprocess
import sys
from src.ui.DialogWindow import ActionDialogWindow

FIX_SELECTION_TEXT = "This will fix imports in Python files.\nAre you sure you want to proceed?"
UNFIX_SELECTION_TEXT = "This will reset imports in Python files.\nAre you sure you want to proceed?"
SKIPPED_FILES = ["__pycache__", "__init__.py", "LICENSE", "README.md", "requirements.txt", "setup.py"]

def fix(filepath):
    ActionDialogWindow("Are you sure?", FIX_SELECTION_TEXT, negativeAction=exit, mainWindow=True)
    filesToParentPackagesMap = getFilesToParentPackagesMap(filepath)
    traverseFoldersAndFixPyImports(filepath, filesToParentPackagesMap)

def getFilesToParentPackagesMap(topLevelFilepath):
    filesToParentPackagesMap = {}
    traverseFoldersAndPopulatePathsMap(topLevelFilepath, filesToParentPackagesMap)
    return filesToParentPackagesMap

def traverseFoldersAndPopulatePathsMap(filepath, filesToParentPackagesMap):
    folderName = os.path.basename(filepath)
    contents = os.listdir(filepath)
    
    for item in contents:
        if item.startswith(".") or item in SKIPPED_FILES:
            continue
        
        newPath = filepath + "/" + item
        if os.path.isdir(newPath):
            traverseFoldersAndPopulatePathsMap(newPath, filesToParentPackagesMap)
        
        else:
            filenameAndExtension = item.split(".")
            if len(filenameAndExtension) != 2:
                continue
            
            filename, extension = filenameAndExtension
            if extension == "py":
                filesToParentPackagesMap[filename] = getFormattedPackageName(filepath)

def getFormattedPackageName(filepath):
    topLevelDirectory = os.path.basename(os.path.dirname(__file__))
    fullPackageArray = filepath.split("/")
    startingIndex = 0
    for index, directory in enumerate(fullPackageArray):
        if directory == topLevelDirectory:
            startingIndex = index
            break
    return ".".join(fullPackageArray[startingIndex:])

def traverseFoldersAndFixPyImports(filepath, filesToParentPackagesMap):
    contents = os.listdir(filepath)
    for item in contents:
        if item.startswith(".") or item in SKIPPED_FILES:
            continue
        
        newPath = filepath + "/" + item
        if os.path.isdir(newPath):
            traverseFoldersAndFixPyImports(newPath, filesToParentPackagesMap)
        
        else:
            filenameAndExtension = item.split(".")
            if len(filenameAndExtension) != 2:
                continue
            
            filename, extension = filenameAndExtension
            if extension == "py":
                fixPyImports(filepath, item, filesToParentPackagesMap)

def fixPyImports(filepath, item, filesToParentPackagesMap):
    currentFolder = os.path.basename(filepath)
    filename = filepath + "/" + item
    with open(filename, "r") as file:
        data = file.readlines()
    
    for index, rawLine in enumerate(data):
        line = rawLine.split()
        if not len(line) or line[0] not in ["import", "from"]:
            continue
        
        if line[0] == "import": 
            # import file 
            # OR import file as name
            importedFile = line[1]
            if importedFile not in filesToParentPackagesMap:
                continue
            
            if filesToParentPackagesMap[importedFile] == currentFolder:
                continue
                
            relativePackage = getRelativePackage(currentFolder, importedFile, 
                filesToParentPackagesMap)
            
            if relativePackage == "":
                continue
            
            if len(line) == 2: 
                # import file
                data[index] = f"from {relativePackage} import {importedFile}\n"
        
            elif len(line) == 4: 
                # import file as name
                restOfImportLine = " ".join(line[2:])
                data[index] = f"from {relativePackage} import {importedFile} {restOfImportLine}\n"
        
        elif line[0] == "from":
            if line[1] not in filesToParentPackagesMap: 
                # from file import function
                continue
            
            importedFile = line[1]
            restOfImportLine = " ".join(line[3:])
            relativePackage = getRelativePackage(currentFolder, importedFile, 
                filesToParentPackagesMap)
            
            if relativePackage == "":
                continue
            
            newFilename = relativePackage + "." + importedFile
            data[index] = f"from {newFilename} import {restOfImportLine}\n"
        
    with open(filename, "w") as file:
        file.writelines(data)

def getRelativePackage(currentFolder, importedFile, filesToParentPackagesMap):
    packageArray = filesToParentPackagesMap[importedFile].split(".")
    startingIndex = 0
    for index, directory in enumerate(packageArray):
        if directory == currentFolder:
            startingIndex = index + 1
    return ".".join(packageArray[startingIndex:])

###################################################################################################
########################################   Unfix   ################################################
###################################################################################################

def unfix(filepath):
    ActionDialogWindow("Are you sure?", UNFIX_SELECTION_TEXT, negativeAction=exit, mainWindow=True)
    filesSet = getFilesSet(filepath)
    traverseFoldersAndUnfixPyImports(filepath, filesSet)

def getFilesSet(topLevelFilepath):
    filesSet = set()
    traverseFoldersAndPopulateFilesSet(topLevelFilepath, filesSet)
    return filesSet

def traverseFoldersAndPopulateFilesSet(filepath, filesSet):
    folderName = os.path.basename(filepath)
    contents = os.listdir(filepath)
    
    for item in contents:
        if item.startswith(".") or item in SKIPPED_FILES:
            continue
        
        newPath = filepath + "/" + item
        if os.path.isdir(newPath):
            traverseFoldersAndPopulateFilesSet(newPath, filesSet)
        
        else:
            filenameAndExtension = item.split(".")
            if len(filenameAndExtension) != 2:
                continue
            
            filename, extension = filenameAndExtension
            if extension == "py":
                filesSet.add(filename)

def traverseFoldersAndUnfixPyImports(filepath, filesSet):
    contents = os.listdir(filepath)
    for item in contents:
        if item.startswith(".") or item in SKIPPED_FILES:
            continue
        
        newPath = filepath + "/" + item
        if os.path.isdir(newPath):
            traverseFoldersAndUnfixPyImports(newPath, filesSet)
        
        else:
            filenameAndExtension = item.split(".")
            if len(filenameAndExtension) != 2:
                continue
            
            filename, extension = filenameAndExtension
            if extension == "py":
                unfixPyImports(filepath, item, filesSet)

def unfixPyImports(filepath, item, filesSet):
    currentFolder = os.path.basename(filepath)
    filename = filepath + "/" + item
    with open(filename, "r") as file:
        data = file.readlines()
    
    for index, rawLine in enumerate(data):
        line = rawLine.split()
        if not len(line) or line[0] not in ["import", "from"]:
            continue
        
        if line[0] == "import": 
            # import file OR import folder.file 
            # OR import file as name 
            # OR import folder.file as name
            if line[1] in filesSet:
                continue
            
            importedFile = line[1].split(".")[-1]
            restOfImportLine = "" if len(line) == 2 else line[2:]
            
            data[index] = f"import {importedFile} {' '.join(restOfImportLine)}\n"
        
        elif line[0] == "from": 
            # from file import function 
            # OR from folder.file import function 
            # OR from folder.folder import file 
            # OR from folder.folder import file as name
            if line[1] in filesSet: 
                # from file import function
                continue
            
            importedFile = line[1]
            importedFile = importedFile.split(".")[-1]
            restOfImportLine = "" if len(line) == 2 else line[2:]
            
            if importedFile in filesSet: 
                # from folder.file import function as name
                data[index] = f"from {importedFile} {' '.join(restOfImportLine)}\n"
            
            elif importedFile not in filesSet: 
                # from folder.folder import file as name
                data[index] = f"{' '.join(restOfImportLine)}\n"
    
    with open(filename, "w") as file:
        file.writelines(data)



import datetime
import os
import subprocess
from src.ui.DialogWindow import ActionDialogWindow

DEFAULT_INITIAL_VERSION = "0.1.0"
UNIT_TEST_CREATION_PROMPT_TEXT = 'Create template unit test files?\nThese will be placed in a \n\
"tests" folder in your project.'

def generate(filepath, projectName, authorsArray, emailsArray, correspondingEmail, githubUsername,
	shortDescription, classifier):
	checkForUnitTests(filepath)
	generateRequirementsFile(filepath)
	generateGitIgnoreFile(filepath)
	generateLicenseFile(filepath, authorsArray)
	generateReadmeFile(filepath, projectName, shortDescription, authorsArray, emailsArray, 
		githubUsername)
	generateInitFiles(filepath)
	generateSetupFile(filepath, authorsArray, correspondingEmail, githubUsername, projectName, 
		shortDescription, classifier)

def checkForUnitTests(filepath):
	if "tests" not in os.listdir(filepath):
		dialogWindow = ActionDialogWindow("No unit tests found", UNIT_TEST_CREATION_PROMPT_TEXT, 
			positiveAction=lambda: createUnitTestTemplates(filepath))

def createUnitTestTemplates(filepath):
	pythonFileFilepaths = []
	appendPythonFileFilepathsToList(filepath, pythonFileFilepaths)
	newPaths = createNewPaths(filepath, pythonFileFilepaths)
	importStrings = createImportStrings(filepath, pythonFileFilepaths)
	createTestFiles(filepath, newPaths, importStrings)

def appendPythonFileFilepathsToList(filepath, pythonFileFilepaths):
	contents = os.listdir(filepath)
	for item in contents:
		if item.startswith(".") or item.startswith("__") or item == "setup.py":
			continue
		
		extension = item.split(".")[-1]
		if extension == "py":
			pythonFileFilepaths.append(filepath + "/" + item)
		
		itemPath = filepath + "/" + item
		if os.path.isdir(itemPath):
			appendPythonFileFilepathsToList(itemPath, pythonFileFilepaths)

def createImportStrings(filepath, pythonFileFilepaths):
	startingIndex = len(filepath.split("/"))
	importStrings = []
	for path in pythonFileFilepaths:
		pathWithoutExtension = path.split(".")[0]
		importStringPath = ".".join(pathWithoutExtension.split("/")[startingIndex:])
		filename = importStringPath.split(".")[-1]
		importString = f"import {importStringPath} as {filename}"
		importStrings.append(importString)
	return importStrings

def createNewPaths(filepath, pythonFileFilepaths):
	startingIndex = len(filepath.split("/"))
	newPaths = []
	for path in pythonFileFilepaths:
		pathArray = path.split("/")
		pathArray[startingIndex] = "tests"
		newPaths.append("/".join(pathArray))
	return newPaths

def createTestFiles(filepath, newPaths, importStrings):
	startingIndex = len(filepath.split("/"))
	for path, importString in zip(newPaths, importStrings):
		directory = "/".join(path.split("/")[:-1])
		moduleName = path.split("/")[-1].split(".")[0].capitalize()
		contents = f"import unittest\n{importString}\n\n# This unit test uses Python's built-in \
unit testing framework\n# See https://docs.python.org/3/library/unittest.html for more information\
\n\nclass Test{moduleName}(unittest.TestCase):\n    def test_condition_doesThis_whenThis(self):\
\n        pass\n\nif __name__ == '__main__':\n    unittest.main()\n"

		subprocess.call(["mkdir", "-p", directory])
		
		with open(path, "w") as file:
			file.write(contents)

def generateRequirementsFile(filepath):
	checkPipreqsInstallation()
	subprocess.run(["pipreqs", filepath], capture_output=True)

def checkPipreqsInstallation():
	try:
		import pipreqs 
	except:
		commandArray = ["pip", "install", "pipreqs"]
		commandArray2 = ["pip3", "install", "pipreqs"]
		try:
			subprocess.call(commandArray)
		except:
			subprocess.call(commandArray2)
		print()

def generateGitIgnoreFile(filepath):
	output = ".DS_Store\n__pycache__/\n*.py[cod]"
	with open(filepath + "/.gitignore", "w") as file:
		file.write(output)

def generateLicenseFile(filepath, authorsArray):
	currentYear = datetime.datetime.now().year
	if len(authorsArray):
		copyrightLine = f"\nCopyright (c) {currentYear} {getAuthorNamesInline(authorsArray)}\n"
	else:
		copyrightLine = ""
	
	output = f'\
MIT License\n{copyrightLine}\
\n\
Permission is hereby granted, free of charge, to any person obtaining a copy \
of this software and associated documentation files (the "Software"), to deal \
in the Software without restriction, including without limitation the rights \
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell \
copies of the Software, and to permit persons to whom the Software is \
furnished to do so, subject to the following conditions:\n\
\n\
The above copyright notice and this permission notice shall be included in all \
copies or substantial portions of the Software.\n\
\n\
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR \
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE \
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, \
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE \
SOFTWARE.'
	
	with open(filepath + "/LICENSE", "w") as file:
		file.write(output)

def getAuthorNamesInline(authorsArray):
	if len(authorsArray) == 1:
		return authorsArray[0]
	
	elif len(authorsArray) == 2:
		return f"{authorsArray[0]} and {authorsArray[1]}"
	
	else:
		allButLast = authorsArray[:-1]
		last = authorsArray[-1]
		return ", ".join(allButLast) + ", and " + last

def generateReadmeFile(filepath, projectName, blurb, authorsArray, emailsArray, githubUsername):
	blurbLine = "" if blurb == "" else f"> {blurb}\n"
	if len(authorsArray):
		namesAndEmails = getNamesAndEmailsVerticallySpaced(authorsArray, emailsArray)
		authorSection = f"## Authors\n\n{namesAndEmails}\n\n"
	else:
		authorSection = ""
	
	if githubUsername == "":
		licenseDetails = ""
		bugReports = ""
	else:
		licenseDetails = f"\
See the [LICENSE](https://github.com/{githubUsername}/{projectName}/blob/main/LICENSE) file for details."
		bugReports = f"\n\
### Bug Reports and Feature Requests\n\
\n\
To report a bug, visit the [issues page](https://github.com/{githubUsername}/{projectName}/issues). \
New feature requests are also welcome!"
	
	output = f"\
# {projectName}\n\
{blurbLine}\
\n\
[![Donate](https://img.shields.io/badge/Donate-PayPal-green.svg)](PAYPAL_DONATE_LINK_GOES_HERE)\n\
\n\
OPTIONAL: ENTER A 2-3 SENTENCE DESCRIPTION ABOUT THE SOFTWARE HERE\n\
\n\
## Getting Started\n\
\n\
### Dependencies\n\
\n\
[Python 3](https://www.python.org/downloads/) must be installed before {projectName} can be installed.\n\
\n\
### Installation\n\
\n\
To install {projectName}, open a terminal window and run the following command:\n\
\n\
```\n\
pip install {projectName}\n\
```\n\
\n\
*(Note that the* `pip3` *command may be required instead of* `pip` *for some Python installations.)*\n\
\n\
### Running the Software\n\
\n\
To launch the main program, run the following command:\n\
\n\
```\n\
{projectName}\n\
```\n\
\n\
## SOME_OTHER_HEADER\n\
\n\
IMAGES INCLUDED BY LINKING TO THEIR LOCATION IN THE PROJECT, E.G.:\n\
![](https://github.com/bdavis222/dotscanner/blob/main/images/3.png)\n\
\n{authorSection}\
## Release History\n\
\n\
* {DEFAULT_INITIAL_VERSION}\n\
	 * Initial Release\n\
\n\
## License\n\
\n\
This project is licensed under the MIT License. {licenseDetails}\n\
\n\
## Development\n\
\n\
To contribute, download or clone the project. From the top level of the project's folder structure, \
one can use the following command to run a local version of the software (e.g., for UI testing):\n\
\n\
```\n\
python -m src\n\
```\n\
\n\
*(Note that the* `python3` *command may be required instead of* `python` *for some Python installations.)*\n\
\n\
### Testing\n\
\n\
Unit tests for this software were written for use with [Python's built-in unittest \
framework](https://docs.python.org/3/library/unittest.html), and are stored in the `tests` folder. \
To run tests, download the project, navigate to the top level of the project's folder structure and \
run the following command:\n\
\n\
```\n\
python -m unittest\n\
```\n\
{bugReports}"
	
	with open(filepath + "/README.md", "w") as file:
		file.write(output)

def getNamesAndEmailsVerticallySpaced(authorsArray, emailsArray):
	namesAndEmails = []
	for name, email in zip(authorsArray, emailsArray):
		if email == "":
			namesAndEmails.append(name)
		else:
			namesAndEmails.append(f"{name} ({email})")
	return "\n\n".join(namesAndEmails)

def generateInitFiles(filepath):
	searchForPyFiles(filepath, includeThisDirectory=False)

def searchForPyFiles(filepath, includeThisDirectory=True):
	pyFileFound = False
	contents = os.listdir(filepath)
	for item in contents:
		if item.startswith("."):
			continue
		extension = item.split(".")[-1]
		if includeThisDirectory and extension == "py" and not pyFileFound:
			createInitFileAt(filepath)
			pyFileFound = True
		
		itemPath = filepath + "/" + item
		if os.path.isdir(itemPath):
			searchForPyFiles(itemPath)

def createInitFileAt(filepath):
	with open(filepath + "/__init__.py", "w") as openFile:
		openFile.write("")

def generateSetupFile(filepath, authorsArray, correspondingEmail, githubUsername, projectName, 
	shortDescription, classifier):
	pyModules = getPyModules(filepath)
	
	authorNameLine = getAuthorsNameLine(authorsArray)
	authorEmailLine = getAuthorEmailLine(correspondingEmail)
	descriptionLine = getDescriptionLine(shortDescription)
	githubInfo = getGithubInfo(githubUsername, projectName)
	classifiers = getClassifiers(classifier)
	
	output = f"from setuptools import setup, find_packages\n\
import pathlib\n\
\n\
here = pathlib.Path(__file__).parent.resolve()\n\
\n\
long_description = (here / 'README.md').read_text(encoding='utf-8')\n\
\n\
setup(\n\
	name='{projectName}', # Required\n\
	version='{DEFAULT_INITIAL_VERSION}', # Required {descriptionLine}\n\
	long_description=long_description,\n\
	long_description_content_type='text/markdown',\
{githubInfo[0]}{authorNameLine}{authorEmailLine}{classifiers}\n\
	packages=find_packages(), # Required\n\
	py_modules={pyModules}, # Generated\n\
	python_requires='>=3.7, <4', {githubInfo[1]}\n\
	entry_points={{\n\
		'console_scripts': [\n\
			'{projectName} = src.__main__:main'\n\
		]\n\
	}}\n\
)"
	with open(filepath + "/setup.py", "w") as file:
		file.write(output)

def getPyModules(filepath):
	pyModules = []
	searchForModules(filepath, filepath, pyModules)
	return pyModules

def searchForModules(absoluteFilepath, filepath, pyModules):
	contents = os.listdir(filepath)
	for item in contents:
		if item.startswith(".") or item.endswith("egg-info") or item in ["build", "dist"]:
			continue
		elif item == "__init__.py":
			pyModules.append(getFormattedPath(absoluteFilepath, filepath))
		
		itemPath = filepath + "/" + item
		if os.path.isdir(itemPath):
			searchForModules(absoluteFilepath, itemPath, pyModules)

def getFormattedPath(absoluteFilepath, filepath):
	topLevelFolderName = absoluteFilepath.split("/")[-1]
	filepathArray = filepath.split("/")
	index = 0
	folderName = filepathArray[index]
	while folderName != topLevelFolderName:
		folderName = filepathArray[index]
		index += 1
	return ".".join(filepathArray[index:])

def getAuthorsNameLine(authorsArray):
	if not len(authorsArray):
		return ""
	else:
		return f"\n    author='{getAuthorNamesInline(authorsArray)}',"

def getAuthorEmailLine(correspondingEmail):
	return f"\n    author_email='{correspondingEmail}'," if correspondingEmail != "" else ""

def getDescriptionLine(description):
	return f"\n    description='{description}'," if description != "" else ""

def getGithubInfo(username, projectName):
	if username == "":
		return ["", ""]
	else:
		urlLine = f"\n    url='https://github.com/{username}/{projectName}',"
		projectUrlsLine = f"\n\
	project_urls={{\n\
		'Bug Reports': 'https://github.com/{username}/{projectName}/issues',\n\
		# 'Funding': 'PAYPAL_DONATE_LINK_GOES_HERE',\n\
		'Source': 'https://github.com/{username}/',\n\
	}},"
		return [urlLine, projectUrlsLine]

def getClassifiers(classifier):
	if classifier in ["N/A", "Select..."]:
		return ""
	else:
		return f"\n\
	classifiers=[ # Defined at https://pypi.org/classifiers/\n\
		'Intended Audience :: {classifier}',\n\
		'License :: OSI Approved :: MIT License',\n\
		'Programming Language :: Python :: 3',\n\
		'Operating System :: Unix',\n\
		'Operating System :: MacOS :: MacOS X',\n\
		'Operating System :: Microsoft :: Windows'\n\
	],"



# import mock 
# import unittest 

# # Below is an example of a test class used for another program. This will not run, as many things 
# # are not actually imported here, and the files themselves are not contained in this template project

# class TestExample(unittest.TestCase):
#     # Test case 1 (just an example of how a test case should be named)
#     def test_condition_doesThis_whenThis(self):
#         pass
    
#     # Test case 2 (with function mocks)
#     @mock.patch("dotscanner.files.os.path.basename") # Mocking a function return value
#     @mock.patch("dotscanner.files.os.path.dirname") # Mocking a function return value
#     @mock.patch("dotscanner.files.os.path.isfile") # Mocking a function return value
#     # Note that the mocks are passed in as parameters IN REVERSE ORDER to the declarations above
#     def test_getDirectoryAndFilenames_getsSingleFile_whenSelectedPathIsFile(self, mock_isfile, mock_dirname, mock_basename):
#         # Convention is to have three code blocks in a test. The first is all of the setup:
#         # Mock return values defined here:
#         mock_isfile.return_value = True
#         mock_dirname.return_value = "test/directory/"
#         mock_basename.return_value = "testFile.png"
#         # Any fake objects needed defined here:
#         fakeUserSettings = FakeUserSettings()
        
#         # The second code block is all of the actions:
#         directory, filenames = files.getDirectoryAndFilenames(fakeUserSettings)
#         # os.path.basename, os.path.dirname, and os.path.isfile are all used somewhere within this 
#         # getDirectoryAndFilenames method, which is why their return values were mocked above
        
#         # The third code block is all of the assertions
#         self.assertEqual(directory, "test/directory/")
#         self.assertEqual(filenames, ["testFile.png"])
    
#     # Test case 3 (with a variable mock)
#     @mock.patch("settings.config.DENSITY_OUTPUT_FILENAME", "density.txt") # Mocking a variable
#     def test_outputString_signifiesSkip_whenFileWasAlreadyMeasured(self):
#         output = strings.alreadyMeasuredNotification(filename="test.png")
        
#         self.assertEqual(
#             output, 
#             f"\nFile test.png already measured in density.txt file. Skipping."
#         )

# if __name__ == "__main__":
#     unittest.main()



import tkinter as tk

class DialogWindow:
	def __init__(self, title, message, positiveButtonText, negativeButtonText, 
		positiveButtonAction=None, negativeButtonAction=None, windowWidth=300, windowHeight=140, 
		positiveButtonColor="blue", negativeButtonColor="black", 
		mainWindow=False):		
		self.window = tk.Tk()
		self.window.title(title)
		centerX = (self.window.winfo_screenwidth() - windowWidth) // 2
		centerY = (self.window.winfo_screenheight() - windowHeight) // 2
		self.window.geometry(f"{windowWidth}x{windowHeight}+{centerX}+{centerY}")
		
		self.positiveButtonAction = positiveButtonAction
		self.negativeButtonAction = negativeButtonAction
		
		spacer = tk.Label(self.window, text=" ")
		messageText = tk.Label(self.window, text=message)
		spacer2 = tk.Label(self.window, text=" ")
		
		buttonsFrame = tk.Frame(self.window)
		
		if positiveButtonAction is None:
			positiveButton = tk.Button(self.window, text=positiveButtonText,
				fg=positiveButtonColor, command=self.closeWindow)
			self.window.bind("<Return>", self.closeWindowWithReturnKey)
		else:
			positiveButton = tk.Button(self.window, text=positiveButtonText, 
				fg=positiveButtonColor, command=self.performPositiveAction)
			self.window.bind("<Return>", self.performPositiveActionWithReturnKey)
		
		if negativeButtonAction is None:
			negativeButton = tk.Button(self.window, text=negativeButtonText, 
				fg=negativeButtonColor, command=self.closeWindow)
			self.window.bind("<Escape>", self.closeWindowWithEscapeKey)
		else:
			negativeButton = tk.Button(self.window, text=negativeButtonText, 
				fg=negativeButtonColor, command=self.performNegativeAction)
			if negativeButtonAction == exit:
				self.window.bind("<Escape>", self.performNegativeActionWithEscapeKey)
		
		negativeButton.pack(in_=buttonsFrame, side=tk.LEFT)
		positiveButton.pack(in_=buttonsFrame, side=tk.LEFT)
		
		spacer.pack()
		messageText.pack()
		spacer2.pack()
		buttonsFrame.pack()
		
		if mainWindow:
			self.window.protocol("WM_DELETE_WINDOW", quit)
		
		self.window.mainloop()
	
	def closeWindow(self):
		self.window.destroy()
		self.window.quit()
	
	def closeWindowWithEscapeKey(self, event):
		self.closeWindow()
	
	def closeWindowWithReturnKey(self, event):
		self.closeWindow()
	
	def performPositiveAction(self):
		self.closeWindow()
		self.positiveButtonAction()
	
	def performPositiveActionWithReturnKey(self, event):
		self.closeWindow()
		self.positiveButtonAction()
	
	def performNegativeAction(self):
		self.closeWindow()
		self.negativeButtonAction()
	
	def performNegativeActionWithEscapeKey(self, event):
		self.closeWindow()
		self.negativeButtonAction()

class ActionDialogWindow(DialogWindow):
	def __init__(self, title, message, positiveAction=None, negativeAction=None, 
		negativeColor="red", mainWindow=False):
		super().__init__(title, message, positiveButtonText="Confirm", negativeButtonText="Cancel",
			positiveButtonAction=positiveAction, negativeButtonAction=negativeAction,
			negativeButtonColor=negativeColor, mainWindow=mainWindow)

class InfoDialogWindow(DialogWindow):
	def __init__(self, title, message, negativeColor="black"):
		super().__init__(title, message, positiveButtonText="Ok", negativeButtonText="Go back", 
			negativeButtonColor=negativeColor)
